"""
generic test harness - runs "all" the test_*.py files and
writes the output to TESTS.OUT-{pyversion}

-- frankm@hiwaay.net
"""

import os,sys
from time import time
from gnosis.xml.pickle.util import enumParsers

# use same python that we're running under
py = sys.executable

# these are ordered (roughly) from simplest->hardest, in terms
# of sophistication of the parser -- ie. when building a new
# parser, you might want to build up capability in this order

# NOTE! Not *ALL* test_*.py files are suitable for a batch run,
# so we list the tests we want
tests = ['test_bltin.py','test_basic.py','test_paranoia.py','test_mixin.py',
         'test_compat.py', 'test_ref.py', 'test_modnames.py',
         'test_ftypes.py', 'test_objectify.py', 'test_getstate.py', 'test_init.py',
         'test_selfref.py', 'test_mutators.py', 'test_re.py','test_rawp_sre.py']

try:
    import gzip
    tests.append('test_4list.py')
except:
    print "** OMITTING test_4list (missing zlib) **"
    
# if mx.DateTime installed, add those tests
try:
    import mx.DateTime
    tests.append('test_mx.py')
    tests.append('test_rawp_mx.py')
except:
    print "** OMITTING test_mx.py & test_rawp_mx.py"

# if Random works, add setstate test
try:
    import random
    r = random.Random()
    tests.append('test_setstate.py')
except:
    print "** OMITTING test_setstate.py"

# if Numeric installed, add numpy test
try:
    import Numeric
    tests.append('test_numpy.py')
except:
    print "** OMITTING test_numpy.py"   

tout = 'TESTS.OUT-%s' % os.path.split(py)[-1]
os.system("rm -f %s; touch %s" %(tout,tout))

# get available parsers
parser_dict = enumParsers()

# test with DOM parser, if available
if parser_dict.has_key('DOM'):

    print "** DOM parser **"
    os.system("echo \"** DOM PARSER **\" >> %s" % tout)
    
    # make sure the USE_.. files are gone
    os.system("rm -f USE_SAX")
    os.system("rm -f USE_HEXPAT")

    t1 = time()

    for test in tests:
        print "Running %s" % test
        os.system("echo \"** %s %s **\" >> %s" % (py,test,tout))
        r = os.system("%s %s >> %s"%(py,test,tout))
        if r != 0:
            print "***ERROR***"
            sys.exit(1)

    print "%.1f seconds" % (time()-t1)
else:
    print "** SKIPPING DOM parser **"
    
# test with SAX parser, if available
if parser_dict.has_key("SAX"):

    print "** SAX parser **"
    os.system("echo \"** SAX PARSER **\" >> %s" % tout)
    
    os.system("touch USE_SAX")

    t1 = time()

    for test in tests:
        print "Running %s" % test
        os.system("echo \"** %s %s **\" >> %s" % (py,test,tout))
        r = os.system("%s %s >> %s"%(py,test,tout))
        if r != 0:
            print "***ERROR***"
            sys.exit(1)

    print "%.1f seconds" % (time()-t1)

    os.system("rm USE_SAX");
else:
    print "** SKIPPING SAX parser **"
    
# test with hEXPAT parser, if available
if parser_dict.has_key("hEXPAT"):
    
    print "** hEXPAT parser **"
    os.system("echo \"** hEXPAT PARSER **\" >> %s" % tout)
    
    os.system("touch USE_HEXPAT");

    t1 = time()

    for test in tests:
        print "Running %s" % test
        os.system("echo \"** %s %s **\" >> %s" % (py,test,tout))
        r = os.system("%s %s >> %s"%(py,test,tout))
        if r != 0:
            print "***ERROR***"
            sys.exit(1)

    print "%.1f seconds" % (time()-t1)      

    os.system("rm -f USE_HEXPAT");
else:
    print "** SKIPPING hEXPAT parser **"
